@@ -69,74 +69,73 @@
 #'
 #' @examples
 #'  library(MASS)
-#'  n = 1000
-#'  alpha1 = matrix(rep(c(2.3, -0.3, -0.01, 0.02, 0.03, 0.04, -0.4),3),ncol=3)
-#'  alpha2 = matrix(rep(c(2.3, -0.3, -0.01, 0.02, 0.03, 0.04, -0.9),4),ncol=4)
-#'  alpha3 = matrix(rep(c(2.3, -0.3, -0.01, 0.02, 0.03, 0.04, -0.9),5),ncol=5)
-#'  beta = c(0.2, -0.3, -0.01, 0.02, 0.03, 0.04, rep(0.02,3),
-#'  rep(0.04,4), rep(0.07,5))
+#'  j<- 500
+#'  p_z <- 6 ## dimension of Z at each time point
+#'  n_t <- 4 ## number of time points
+#'  alphas <- list()
+#'  gammas <- list()
+#'  z_para <- c(-1/p_z, -1/p_z, -1/p_z, -1/p_z, -0.5/p_z,-0.5/p_z, -0.5/p_z,
+#'  -0.5/p_z)
+#'  Z <- list()
+#'  beta = c(0.2, -0.3, -0.01, 0.02, 0.03, 0.04, rep(rep(0.02,p_z), n_t))
 #'  beta_T = -0.2
-#'  gamma1 = c(1, -0.1, 0.2, 0.2, 0.2, 0.2, rep(-1/3,3))     #setting 1
-#'  gamma2 = c(1, -0.1, 0.2, 0.2, 0.2, 0.2, rep(-2/4,4))     #setting 1
-#'  gamma3 = c(1, -0.1, 0.2, 0.2, 0.2, 0.2, rep(-2.5/5,5))   #setting 1
 #'  sd_z_x = 0.4
-#'  X = mvrnorm(n, mu=c(1,5,6,7,8), Sigma=diag(1,5))
-#'  TRT = rbinom(n, size = 1,  prob = 0.5)
-#'  Z0_1 = alpha1[1,]+(X%*%alpha1[2:6,])           + mvrnorm(n, mu = rep(0,3),
-#'  Sigma = diag(sd_z_x,3))
-#'  Z1_1 = alpha1[1,]+(X%*%alpha1[2:6,])+alpha1[7,] + mvrnorm(n, mu = rep(0,3),
-#'  Sigma = diag(sd_z_x,3))
-#'  Z_1  = Z1_1 * TRT+Z0_1 * (1-TRT)
+#'  X = mvrnorm(j, mu=c(1,5,6,7,8), Sigma=diag(1,5))
+#'  TRT = rbinom(j, size = 1,  prob = 0.5)
+#'  Y_constant <- beta[1]+(X%*%beta[2:6])
+#'  Y0 <- 0
+#'  Y1 <- 0
+#'  A <- A1 <- A0 <- matrix(NA, nrow = j, ncol = n_t)
+#'  gamma <- c(1,-.1,-0.05,0.05,0.05,.05)
+#'  A0[,1] <- rbinom(j, size = 1, prob = 1/(1+exp(-(gamma[1] +
+#'  (X %*% gamma[2:6])))))
+#'  A1[,1] <- rbinom(j, size = 1, prob = 1/(1+exp(-(gamma[1] +
+#'  (X %*% gamma[2:6])))))
+#'  A[,1] <- A1[,1]*TRT + A0[,1]*(1-TRT)
+
+#'  for(i in 2:n_t){
+#'    alphas[[i]] <- matrix(rep(c(2.3, -0.3, -0.01, 0.02, 0.03, 0.04, -0.4),
+#'    p_z),ncol=p_z)
+#'    gammas[[i]] <- c(1, -0.1, 0.2, 0.2, 0.2, 0.2, rep(z_para[i],p_z))
+#'    Z0 <- alphas[[i]][1,]+(X%*%alphas[[i]][2:6,]) + mvrnorm(j, mu = rep(0,p_z)
+#'    , Sigma = diag(sd_z_x,p_z))
+#'    Z1 <- alphas[[i]][1,]+(X%*%alphas[[i]][2:6,])+alphas[[i]][7,] +
+#'      mvrnorm(j, mu = rep(0,p_z), Sigma = diag(sd_z_x,p_z))
+#'    Z[[i]] <- Z1*TRT + Z0*(1-TRT)
+#'    Y0 <- (Y0 + Z0 %*% matrix(beta[ (7 + (i-1)*p_z):
+#'    (6+p_z*i)],ncol = 1) )[,1]
+#'    Y1 <- (Y1 + Z1 %*% matrix(beta[ (7 + (i-1)*p_z):
+#'    (6+p_z*i)],ncol = 1) )[,1]
+#'    A0[,i] <- rbinom(j, size = 1,
+#'                     prob = 1/(1+exp(-(gammas[[i]][1]+
+#'                     (X%*%gammas[[i]][2:6])+Z0%*%matrix(gammas[[i]][7:
+#'                     (7+p_z-1)], ncol=1))[,1])))*A0[,i-1]
+#'    A1[,i] <- rbinom(j, size = 1,
+#'                     prob = 1/(1+exp(-(gammas[[i]][1]+
+#'                     (X%*%gammas[[i]][2:6])+Z1%*%matrix(gammas[[i]][7:
+#'                     (7+p_z-1)], ncol=1))[,1])))*A1[,i-1]
 #'
-#'  Z0_2 = alpha2[1,]+(X%*%alpha2[2:6,])           + mvrnorm(n, mu = rep(0,4),
-#'  Sigma = diag(sd_z_x,4))
-#'  Z1_2 = alpha2[1,]+(X%*%alpha2[2:6,])+alpha2[7,] + mvrnorm(n, mu = rep(0,4),
-#'  Sigma = diag(sd_z_x,4))
-#'  Z_2  = Z1_2 * TRT + Z0_2 * (1-TRT)
-#'
-#'  Z0_3 = alpha3[1,]+(X%*%alpha3[2:6,])           + mvrnorm(n, mu = rep(0,5),
-#'  Sigma = diag(sd_z_x,5))
-#'  Z1_3 = alpha3[1,]+(X%*%alpha3[2:6,])+alpha3[7,] + mvrnorm(n, mu = rep(0,5),
-#'  Sigma = diag(sd_z_x,5))
-#'  Z_3  = Z1_3 * TRT + Z0_3 * (1-TRT)
-#'  Z = list(Z_1, Z_2, Z_3)
-#'  Y0 = (beta[1]+(X %*% beta[2:6]) + Z0_1 %*% matrix(beta[7:9], ncol = 1) +
-#'  Z0_2 %*% matrix(beta[10:13], ncol = 1) + Z0_3 %*% beta[14:18] +
-#'  rnorm(n, mean = 0, sd = 0.3))[,1]
-#'  Y1 = (beta[1] + (X %*% beta[2:6]) + Z1_1 %*% matrix(beta[7:9], ncol = 1) +
-#'  Z1_2 %*% matrix(beta[10:13], ncol = 1) + Z1_3 %*% beta[14:18] + beta_T +
-#'  rnorm(n, mean = 0, sd = 0.3))[,1]
-#'  Y  = Y1 * TRT + Y0 * (1 - TRT)
-#'
-#'  A0_1 = rbinom(n, size = 1, prob = 1 / (1 + exp(-(gamma1[1] +
-#'  (X %*% gamma1[2:6]) + Z0_1 %*% matrix(gamma1[7:9], ncol = 1))[,1])))
-#'  A1_1 = rbinom(n, size = 1, prob = 1/(1 + exp(-(gamma1[1] +
-#'  (X %*% gamma1[2:6]) + Z1_1 %*% matrix(gamma1[7:9], ncol = 1))[,1])))
-#'  A_1  = A1_1 * TRT + A0_1 * (1 - TRT)
-#'
-#'  A0_2 = rbinom(n, size = 1, prob = 1/(1 + exp(-(gamma2[1] +
-#'  (X %*% gamma2[2:6]) + Z0_2 %*% matrix(gamma2[7:10], ncol = 1))[,1]))) * A0_1
-#'  A1_2 = rbinom(n, size = 1, prob = 1/(1 + exp(-(gamma2[1] +
-#'  (X %*% gamma2[2:6]) + Z1_2 %*% matrix(gamma2[7:10], ncol = 1))[,1]))) * A1_1
-#'  A_2  = A1_2 * TRT + A0_2 * (1 - TRT)
-#'
-#'  A0_3 = rbinom(n, size = 1, prob = 1/(1 + exp(-(gamma3[1] +
-#'  (X %*% gamma3[2:6]) + Z0_3 %*% matrix(gamma3[7:11], ncol = 1))[,1]))) * A0_2
-#'  A1_3 = rbinom(n, size = 1, prob = 1/(1 + exp(-(gamma3[1] +
-#'  (X %*% gamma3[2:6]) + Z1_3 %*% matrix(gamma3[7:11], ncol = 1))[,1]))) * A1_2
-#'  A_3  = A1_3 * TRT + A0_3 * (1 - TRT)
-#'  A = cbind(A_1, A_2, A_3)
-#'
-#'  Z[[2]][A_1 == 0] <- NA
-#'  Z[[3]][A_2 == 0] <- NA
-#'  Y[A_3 == 0]   <- NA
+#'    A[,i] <- A1[,i]*TRT + A0[,i]*(1-TRT)
+#'  }
+#'  Y0 <- Y0 + rnorm(j, mean = 0, sd = 0.3) + Y_constant
+#'  Y1 <- Y1 + + beta_T + rnorm(j, mean = 0, sd = 0.3) + Y_constant
+
+#'  Y <- as.vector( Y1*TRT+Y0*(1-TRT))
+
+#'  for(i in 2:n_t){
+#'    Z[[i]][A[,(i-1)]==0,] <- NA
+#'  }
+
+#'  Z[[1]] <- matrix(NA, nrow=nrow(Z1),ncol=ncol(Z1))
+
+#'  Y[A[,n_t] == 0] <- NA
 #'  # estimate the treatment difference
 #'  fit <- est_S_Star_Plus_MethodA(X, A, Z, Y, TRT)
 #'  fit
 #'  # Calculate the true values
-#'  true1 =  mean(Y1[A1_3 == 1])
+#'  true1 <- mean(Y1[A1[,n_t]==1])
 #'  true1
-#'  true0 =  mean(Y0[A1_3 == 1])
+#'  true0 <- mean(Y0[A1[,n_t]==1])
 #'  true0
 #'  true_d  =  true1 - true0
 #'  true_d
@@ -156,7 +155,7 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
   for (i in 2:n_time_points) {
     Z_cp[[i]][A[, (i - 1)] == 0] <- 0
   }
-  Z_mat <- matrix(unlist(Z), nrow = dim(X)[1]) # nolint
+  Z_mat <- matrix(unlist(Z[-1]), nrow = dim(X)[1]) # nolint
 
   # Provide column names for X, Z, A
   X_col_names <- paste("X_", 1:dim(X)[2], sep = "")   # nolint
@@ -164,11 +163,11 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
   colnames(X) <- X_col_names
   colnames(A) <- A_col_names
   Z_col_names <- list()                               # nolint
-  for (i in 1:n_time_points) {
+  for (i in 2:n_time_points) {
     part1 <- paste("Z_", i, sep = "")
     Z_col_names[[i]] <- paste(part1, 1:dim(Z[[i]])[2], sep = "")
   }
-  colnames(Z_mat) <- unlist(Z_col_names)
+  colnames(Z_mat) <- unlist(Z_col_names[-1])
   data <- data.frame(X, TRT, Z_mat, Y, A)
 
   # Model adherence given X, Z using a logistic model
@@ -196,18 +195,19 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
   # Model Z given X and estimate variance-covariance of Z given X
   models_Z_X <- list()                                            # nolint
   sigma_mats_Z <- list()                                          # nolint
-  for (i in 1:n_time_points) {
+  for (i in 2:n_time_points) {
     models_Z_X[[i]] <- lm(Z_mat[, Z_col_names[[i]]] ~ X + TRT)
     if (dim(Z[[i]])[2] > 1) {
-      sigma_mats_Z[[i]] <- diag(apply(models_Z_X[[i]]$residuals, 2, var))
+      sigma_mats_Z[[i]] <- cov(models_Z_X[[i]]$residuals)
     } else {
-      sigma_mats_Z[[i]] <- matrix(var(models_Z_X[[i]]$residuals), 1, 1)
+      sigma_mats_Z[[i]] <- matrix(var(models_Z_X[[i]]$residuals),
+                                  1, 1)
     }
   }
 
   # Predict Z using models.Z.X
   Zs_pred <- list()                                             # nolint
-  for (i in 1:n_time_points) {
+  for (i in 2:n_time_points) {
 
     Zs_pred[[i]] <- predict(models_Z_X[[i]],
                             newdata = data.frame(X, TRT = rep(1, n)))
@@ -215,19 +215,19 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
 
   # Estimate alpha of Z_i given T for 1 (intercept) and Xs
   coefs_Z_1X <- list()                                         # nolint
-  for (i in 1:n_time_points) {
+  for (i in 2:n_time_points) {
     coef_raw <- matrix(coef(models_Z_X[[i]]), ncol = dim(Z[[i]])[2])
     coefs_Z_1X[[i]] <- matrix(coef_raw[1:(dim(X)[2]+1), ],      # nolint
                               ncol = dim(Z[[i]])[2])
     coefs_Z_1X[[i]][1, ] <- coefs_Z_1X[[i]][1, ] + coef_raw[nrow(coef_raw), ]
   }
   # Calculate required integrations and assigning names for each column
-  Expect_res <- apply(X, 1, Expect_function1D_BU, n_time_points = n_time_points,    # nolint
+  Expect_res <- apply(X, 1, Expect_function1D_BU, n_time_points = n_time_points,
                       gammas = coefs_A_XZ, alphas = coefs_Z_1X,
                       Sigmas = sigma_mats_Z)
-  Expect_res_t <- t(Expect_res)                                                     # nolint
+  Expect_res_t <- t(Expect_res)
   names_vec <- NULL
-  for (i in 1:n_time_points) {
+  for (i in 2:n_time_points) {
     z_dim <- dim(Z[[i]])[2]
     names_prob <- paste("prob", i, sep = "")
     names_expa <- paste("expa", i, sep = "")
@@ -239,6 +239,12 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
   }
   colnames(Expect_res_t) <- names_vec
 
+  prob1_expa1 <- cbind(preds_A_XZ[[1]],preds_A_XZ[[1]]*(1-preds_A_XZ[[1]]))
+  colnames(prob1_expa1) <- c("prob1","expa1")
+  Expect_res_t <- cbind(Expect_res_t,prob1_expa1)
+
+  names_vec <- c(names_vec, c("prob1","expa1"))
+
   # Calculate the probability of adherence at the end of study
   probs_vec <- paste("prob", 1:n_time_points, sep = "")
   Expect_probs <- Expect_res_t[, probs_vec]                         # nolint
@@ -251,20 +257,25 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
   Expect_AZ_X <- list()   # nolint
   Expect_AA_X <- list()   # nolint
   Expect_AA_Z_X <- list()   # nolint
-  for (i in 1:n_time_points) {
+  for (i in 2:n_time_points) {
     names_target <- paste(c("expz", "expa", "expaz"), i, sep = "")
     prob_remove <- paste("prob", i, sep = "")
     Expect_AZ_X[[i]] <- Expect_A_X *
       Expect_res_t[, grepl(names_target[1], names_vec)] /
       Expect_res_t[, prob_remove]
-    Expect_AA_X[[i]] <- Expect_A_X *
-      Expect_res_t[, grepl(names_target[2], names_vec)] /
-      Expect_res_t[, prob_remove]
     Expect_AA_Z_X[[i]] <-  Expect_A_X *
       Expect_res_t[, grepl(names_target[3], names_vec)] /
       Expect_res_t[, prob_remove]
   }
 
+  for (i in 1:n_time_points) {
+    names_target <- paste(c("expz", "expa", "expaz"), i, sep = "")
+    prob_remove <- paste("prob", i, sep = "")
+    Expect_AA_X[[i]] <- Expect_A_X *
+      Expect_res_t[, grepl(names_target[2], names_vec)] /
+      Expect_res_t[, prob_remove]
+  }
+
   # Calcuate a point estimation for S_tar_plus
   preds_A_XZ_clean <- lapply(preds_A_XZ, NA_replace)        # nolint
   prod_A <- 1                    # nolint
@@ -273,7 +284,7 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
   }
   res_main <- sum(TRT * A[, n_time_points] * Y - (1 - TRT) *
                     A[, n_time_points] *
-                    Expect_A_X*Y / prod_A) / sum(TRT * A[, n_time_points])    # nolint
+                    Expect_A_X*Y / prod_A) / sum(TRT * A[, n_time_points])
   residual <- (sum((1 - TRT) * A[, n_time_points] * Expect_A_X * Y / prod_A) /
                  sum(TRT * A[, n_time_points])) * (1 - sum(TRT) / sum(1 - TRT))
   estimator <- res_main + residual
@@ -283,11 +294,13 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
 
   # At each time point, for each dimension of Z, we have an estimation equation
   # g2 = X(Z_i - X\alpha), thus there will be layers of for loop.
-  covariate_long <- lapply(models_Z_X, model.matrix.lm)
+  covariate_long <- vector("list",n_time_points)
+  covariate_long[2:n_time_points] <- lapply(models_Z_X[-1], model.matrix.lm)
+
   g2 <- list()
-  for (i in 1:n_time_points) {
+  for (i in 2:n_time_points) {
     g2_main <- (models_Z_X[[i]]$model$`Z_mat[, Z_col_names[[i]]]`
-                 - predict(models_Z_X[[i]]))
+                - predict(models_Z_X[[i]]))
     if (dim(Z[[i]])[2] > 1) {
       dim_z <- dim(g2_main)[2]
       res <- list()
@@ -305,7 +318,7 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
   g3 <- list()
   for (i in 1:n_time_points) {
     if (i == 1) {
-      g3[[i]] <- (A[, i] - preds_A_XZ_clean[[i]]) * cbind(rep(1, n), X, Z[[i]])
+      g3[[i]] <- (A[, i] - preds_A_XZ_clean[[i]]) * cbind(rep(1, n), X)
     } else {
       g3[[i]] <- A[, (i - 1)] * (A[, i] - preds_A_XZ_clean[[i]]) *
         cbind(rep(1, n), X, Z[[i]])
@@ -325,7 +338,7 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
   # Estimating expection of deriatives using sample averages
   partial_g4_alpha <- list()
   part1_partial <- (1 - TRT) * A[, n_time_points] * Y / (prod_A)
-  for (i in 1:n_time_points) {
+  for (i in 2:n_time_points) {
     dim_z <- dim(Z[[i]])[2]
     partial_vec <- matrix(part1_partial * (Expect_AZ_X[[i]] - Expect_A_X *
                                              Zs_pred[[i]]), ncol = dim_z)
@@ -333,7 +346,8 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
     partial_g4_alpha_x <- matrix(rep(NA, dim_z * dim(X)[2]), ncol = dim_z)
     for (j in 1:dim_z) {
       partial_g4_alpha_x[, j] <- apply(t(X) %*%
-                                 diag(as.vector(partial_vec[, j])), 1, mean)
+                                         diag(as.vector(partial_vec[, j])), 1,
+                                       mean)
     }
     partial_g4_alpha[[i]] <- -solve(sigma_mats_Z[[i]]) %*%
       t(rbind(partial_g4_alpha_1, partial_g4_alpha_x, partial_g4_alpha_1))
@@ -341,33 +355,49 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
 
   partial_g4_gamma <- list()
   for (i in 1:n_time_points) {
-    constant <- (1 - TRT) * A[, n_time_points] * Y /
-      (prod_A / preds_A_XZ_clean[[i]])
-    vec1 <- (Expect_AA_X[[i]] * preds_A_XZ_clean[[i]] -
-               Expect_A_X * preds_A_XZ_clean[[i]] *
-               (1 - preds_A_XZ_clean[[i]])) / (preds_A_XZ_clean[[i]]^2)
-    vec2 <- (Expect_AA_X[[i]] * preds_A_XZ_clean[[i]] -
-               Expect_A_X * preds_A_XZ_clean[[i]] *
-               (1 - preds_A_XZ_clean[[i]])) / (preds_A_XZ_clean[[i]]^2)
-    vec3 <- ((Expect_AA_Z_X[[i]] * preds_A_XZ_clean[[i]]) -
-               Expect_A_X * preds_A_XZ_clean[[i]] *
-              (1 - preds_A_XZ_clean[[i]]) * Z_cp[[i]]) /
-              (preds_A_XZ_clean[[i]]^2)
-    partial_g4_gamma[[i]] <- -c(mean(constant * vec1),
-                                apply(t(X) %*%
-                                diag(as.vector(constant * vec2)), 1, mean),
-                                apply(constant * vec3, 2, mean))
+    if (i == 1) {
+      constant <- (1 - TRT) * A[, n_time_points] * Y /
+        (prod_A / preds_A_XZ_clean[[i]])
+      vec1 <- (Expect_AA_X[[i]] * preds_A_XZ_clean[[i]] -
+                 Expect_A_X * preds_A_XZ_clean[[i]] *
+                 (1 - preds_A_XZ_clean[[i]])) / (preds_A_XZ_clean[[i]]^2)
+      vec2 <- (Expect_AA_X[[i]] * preds_A_XZ_clean[[i]] -
+                 Expect_A_X * preds_A_XZ_clean[[i]] *
+                 (1 - preds_A_XZ_clean[[i]])) / (preds_A_XZ_clean[[i]]^2)
+      partial_g4_gamma[[i]] <- -c(mean(constant * vec1),
+                                  apply(t(X) %*%
+                                          diag(as.vector(constant * vec2)), 1,
+                                        mean))
+    } else {
+      constant <- (1 - TRT) * A[, n_time_points] * Y /
+        (prod_A / preds_A_XZ_clean[[i]])
+      vec1 <- (Expect_AA_X[[i]] * preds_A_XZ_clean[[i]] -
+                 Expect_A_X * preds_A_XZ_clean[[i]] *
+                 (1 - preds_A_XZ_clean[[i]])) / (preds_A_XZ_clean[[i]]^2)
+      vec2 <- (Expect_AA_X[[i]] * preds_A_XZ_clean[[i]] -
+                 Expect_A_X * preds_A_XZ_clean[[i]] *
+                 (1 - preds_A_XZ_clean[[i]])) / (preds_A_XZ_clean[[i]]^2)
+      vec3 <- ((Expect_AA_Z_X[[i]] * preds_A_XZ_clean[[i]]) -
+                 Expect_A_X * preds_A_XZ_clean[[i]] *
+                 (1 - preds_A_XZ_clean[[i]]) * Z_cp[[i]]) /
+        (preds_A_XZ_clean[[i]]^2)
+      partial_g4_gamma[[i]] <- -c(mean(constant * vec1),
+                                  apply(t(X) %*%
+                                          diag(as.vector(constant * vec2)), 1,
+                                        mean),
+                                  apply(constant * vec3, 2, mean))
+    }
   }
 
   partial_g2_alpha <- list()
-  for (i in 1:n_time_points) {
+  for (i in 2:n_time_points) {
     partial_g2_alpha[[i]] <- -t(covariate_long[[i]]) %*% covariate_long[[i]] / n
   }
 
   partial_g3_gamma <- list()
   for (i in 1:n_time_points) {
     if (i == 1) {
-      design_mat <- cbind(rep(1, n), X, Z[[i]])
+      design_mat <- cbind(rep(1, n), X)
       partial_g3_gamma[[i]] <- -t(design_mat) %*% (
         design_mat * preds_A_XZ_clean[[i]] * (1 - preds_A_XZ_clean[[i]])) / n
     } else {
@@ -375,7 +405,8 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
       design_mat_sub <- design_mat[A[, (i - 1)] != 0, ]
       A_pred_sub <- preds_A_XZ_clean[[i]][A[, (i - 1)] != 0]  # nolint
       partial_g3_gamma[[i]] <- -t(design_mat_sub) %*% (design_mat_sub *
-                                A_pred_sub * (1 - A_pred_sub)) / n     # nolint
+                                                         A_pred_sub *
+                                                         (1 - A_pred_sub)) / n
     }
   }
 
@@ -383,8 +414,8 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
   tau_est <- estimator
   data_long <- cbind(data, subj = seq_len(nrow(data)))
   data_long <- reshape2::melt(data_long, id.vars = c(X_col_names, "TRT", "Y",
-                                                    A_col_names, "subj"),
-                             variable.name = "Z")
+                                                     A_col_names, "subj"),
+                              variable.name = "Z")
   temp <- list()
   temp[[1]] <- g2[[1]]
   for (i in 2:n_time_points) {
@@ -394,19 +425,19 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
       for (j in 1:dim_z) {
         res[[j]] <- matrix(0, nrow = n, ncol = ncol(g2[[i]][[j]]))
         res[[j]][!is.na(data_long$value[data_long$Z == Z_col_names[[i]][j]]),
-                 ] <- g2[[i]][[j]]
+        ] <- g2[[i]][[j]]
       }
       temp[[i]] <- res
     } else {
       temp[[i]] <- matrix(0, nrow = n, ncol = ncol(g2[[i]]))
       temp[[i]][!is.na(data_long$value[data_long$Z == Z_col_names[[i]]]),
-                ] <- g2[[i]]
+      ] <- g2[[i]]
     }
 
   }
 
   se_main <- 0
-  for (i in 1:n_time_points) {
+  for (i in 2:n_time_points) {
     dim_z <- dim(Z[[i]])[2]
     if (dim_z > 1) {
       for (j in 1:dim_z) {
@@ -434,11 +465,11 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
   res1 <- sum(TRT * A[, n_time_points] * Y) / sum(TRT * A[, n_time_points])
   res0 <- sum((1 - TRT) * A[, n_time_points] * Expect_A_X * Y / prod_A) /
     sum(TRT * A[, n_time_points]) * (sum(TRT) / sum(1 - TRT))
-  se1  <-  sd(1 / mean(g5) * (TRT * A[, n_time_points] * Y - res1 * g5)) /
+  se_res1  <-  sd(1 / mean(g5) * (TRT * A[, n_time_points] * Y - res1 * g5)) /
     sqrt(n)
 
   se_main_se0 <- 0
-  for (i in 1:n_time_points) {
+  for (i in 2:n_time_points) {
     dim_z <- dim(Z[[i]])[2]
     if (dim_z > 1) {
       for (j in 1:dim_z) {
@@ -458,14 +489,14 @@ est_S_Star_Plus_MethodB <- function(X, A, Z, Y, TRT){                  # nolint
   }
 
   se0 <- se_main_se0 + ((1 - TRT) * A[, n_time_points] * Expect_A_X * Y /
-                          (prod_A) - res0 * g5 - se_residual) / mean(g5)
-  se0 <- sd(se0) / sqrt(n)
+                          (prod_A) - res0 * g5) / mean(g5)
+  se_res0 <- sd(se0) / sqrt(n)
   RVAL <- list(trt_diff = estimator,                                  # nolint
              se = se,
              res1 = res1,
              res0 = res0,
-             se_res1 = se1,
-             se_res0 = se0
+             se_res1 = se_res1,
+             se_res0 = se_res0
   )
   return(RVAL)
 }
